`att` <-
function (obj, formula, data, model="lm",family="binomial") 
{
 if((class(obj)[1] != "cem.match") && (class(obj)[1] != "multicem"))
  stop("Argument `obj' must be a `cem.match' or `multicem' object") 
 if(!(model %in% c("lm","glm")))
  stop("please, only use `lm' or `glm'")
   
 if(class(obj)[1] == "cem.match"){
  if(model=="lm")
   out <- do.call(model, list(formula=formula, data=data, weights=obj$w))
  else
   out <- do.call(model, list(formula=formula, data=data, weights=obj$w,family=family))
  aa <- summary(out)
  out <- list(model = t(aa$coefficients))
  class(out) <- "cem.att"
  return(out)
 }
 
 if(class(data) != "list")
  stop("Argument `data' must be a list of `data.frame's")

 n.cems <- length(obj) - 1

 if(length(data) != n.cems)
  stop("lengths of `multicem' object and `data' do not match")
  
 est <- vector(n.cems, mode="list")
 for(i in 1:n.cems){ 
  if(model=="lm")
   out <- do.call(model, list(formula=formula, data=data[[i]], weights=obj[[i]]$w))
  else
   out <- do.call(model, list(formula=formula, data=data[[i]], weights=obj[[i]]$w,family=family))
  aa <- summary(out)
  est[[i]] <- aa$coefficients 
 }
 qoi <- numeric(dim(est[[1]])[1])
 seq <- numeric(dim(est[[1]])[1])
 for(i in 1:n.cems){
  qoi <- qoi + est[[i]][,1]
  seq <- seq + est[[i]][,2]^2
 }
 
 qoi <- qoi/n.cems
 seq <- seq/n.cems

 s2 <- numeric(dim(est[[1]])[1])
 for(i in 1:n.cems){
  s2 <- s2 + (est[[i]][,1] - qoi)^2
 }
 s2 <- s2/(n.cems-1)

 S <- sqrt(seq + s2*(1+1/n.cems))

 mat <- rbind(qoi, S)
 rownames(mat) <- c("Estimate", "Std. Error")
 
 out <- list(mult=est, model = mat) 
 class(out) <- "cem.att"
 out
}



print.cem.att <- function(x,...){
 if(!is.null(x$tab)){
  cat("\n")
  print(x$tab)
  }
 if(!is.null(x$model)){
  cat("\nEstimate on the CEM matched subsample:\n\n")
  print(x$model)
  }
 if(!is.null(x$bayes)){
  cat("\nBayes estimate on the whole sample:\n\n")
  print(x$bayes)
  }
  
}


############### THE BAYES STUFF ##############


`att.bayes` <-
function (obj, formula, data, model="lm",family="binomial", use.bayes=TRUE, use.all=TRUE) 
{
 require(bayesm)
 if((class(obj)[1] != "cem.match") && (class(obj)[1] != "multicem"))
  stop("Argument `obj' must be a `cem.match' or `multicem' object") 
 if(!(model %in% c("lm","glm")))
  stop("please, only use `lm' or `glm'")
 bayes <- NULL  
 if(class(obj)[1] == "cem.match"){
  if(model=="lm"){
   out <- do.call(model, list(formula=formula, data=data, weights=obj$w))
   if(use.bayes){
    p <- obj$tab[2,2]/obj$tab[1,2]
    att1 <- as.numeric(coef(out)[mat$treatment])

   cemID <- unique(obj$strata[which(obj$matched==FALSE)])
   n.st <- length(cemID)
   regdata <- vector(n.st, mode="list")
   mtx <- model.matrix(formula, data=data)
   y <- model.response(model.frame(formula, data=data))   
   trt <- which(colnames(mtx) == mat$treatment)
   for(s in 1:n.st){
    idx <- which(obj$strata==s)
	if(length(idx)==1)
	 tmp <- matrix(mtx[idx,-1], nrow=1)
	else
	 tmp <- matrix(mtx[idx,-1], nrow=length(idx), ncol=NCOL(mtx)-1, byrow=T) 
    regdata[[s]] <- list(y=matrix(y[idx],length(idx), 1), X=tmp)
   }
   Z <- matrix(1, n.st, 1)
   nvar <- length(attr(terms(formula),"term.labels"))
   Deltabar=matrix(0,1,nvar)
   Deltabar[1,trt-1] <- att1
   
   Data1=list(regdata=regdata, Z=Z)
   Mcmc1=list(R=100,keep=1)

   Prior1=list(Deltabar=Deltabar)
   out.bayes=rhierLinearModel(Data=Data1,Mcmc=Mcmc1, Prior=Prior1)

   att3 <- rowMeans(colMeans(out.bayes$betadraw)) ## this should be the final estimate


   bayes <- att3*(1-p) + p*coef(out)[-1]

}
  } else {
   out <- do.call(model, list(formula=formula, data=data, weights=obj$w,family=family))
  }
  aa <- summary(out)
  out <- list(model = t(aa$coefficients), bayes=bayes, tab=obj$tab)
  class(out) <- "cem.att"
  return(out)
 }
 
 if(class(data) != "list")
  stop("Argument `data' must be a list of `data.frame's")

 n.cems <- length(obj) - 1

 if(length(data) != n.cems)
  stop("lengths of `multicem' object and `data' do not match")
  
 est <- vector(n.cems, mode="list")
 for(i in 1:n.cems){ 
  if(model=="lm")
   out <- do.call(model, list(formula=formula, data=data[[i]], weights=obj[[i]]$w))
  else
   out <- do.call(model, list(formula=formula, data=data[[i]], weights=obj[[i]]$w,family=family))
  aa <- summary(out)
  est[[i]] <- aa$coefficients 
 }
 qoi <- numeric(dim(est[[1]])[1])
 seq <- numeric(dim(est[[1]])[1])
 for(i in 1:n.cems){
  qoi <- qoi + est[[i]][,1]
  seq <- seq + est[[i]][,2]^2
 }
 
 qoi <- qoi/n.cems
 seq <- seq/n.cems

 s2 <- numeric(dim(est[[1]])[1])
 for(i in 1:n.cems){
  s2 <- s2 + (est[[i]][,1] - qoi)^2
 }
 s2 <- s2/(n.cems-1)

 S <- sqrt(seq + s2*(1+1/n.cems))

 mat <- rbind(qoi, S)
 rownames(mat) <- c("Estimate", "Std. Error")
 
 out <- list(mult=est, model = mat) 
 class(out) <- "cem.att"
 out
}

